const { cloneObj } = require('../Util');

class Frequency {
  static groupFreq = {};
  static dishFreq = {};
  static frequencies = {};

  get groupFreq() {
    return Frequency.groupFreq;
  }

  get dishFreq() {
    return Frequency.dishFreq;
  }

  get frequencies() {
    return Frequency.frequencies;
  }

  load(frequencies) {
    Frequency.frequencies = frequencies;
    frequencies.forEach((freq, idx) => {
      const { cur = 0, curWeekUsage = {} } = freq;
      if(freq.NUM_GRP_PRATO) {
        const {
          MIN_MES_GRP, MAX_MES_GRP, MAX_SEM_GRP,
        } = freq;
        Frequency.groupFreq[freq.NUM_GRP_PRATO] = {
          idx,
          week: cloneObj(curWeekUsage),
          month: cur,
          minMonth: Number(MIN_MES_GRP),
          maxMonth: Number(MAX_MES_GRP),
          maxWeek: Number(MAX_SEM_GRP),
        }
      } else if(freq.COD_PRATO) {
        const { MIN_MES_PRATO, MAX_MES_PRATO, MAX_SEMANA } = freq;
        Frequency.dishFreq[freq.COD_PRATO] = {
          idx,
          week: cloneObj(curWeekUsage),
          month: cur,
          minMonth: Number(MIN_MES_PRATO),
          maxMonth: Number(MAX_MES_PRATO),
          maxWeek: Number(MAX_SEMANA),
        }
      }
    });
  }

  addFrequency(dish, weekNumber) {
    const groupFreqInfo = Frequency.groupFreq[dish.gpprato];
    const dishFreqInfo = Frequency.dishFreq[dish.prato];
    if (groupFreqInfo) {
      groupFreqInfo.month++;
      groupFreqInfo.week[weekNumber]++;
      let freq = Frequency.frequencies[groupFreqInfo.idx];
      freq.cur = freq.cur || 0;
      freq.cur++;
      freq.curWeekUsage = freq.curWeekUsage || {};
      freq.curWeekUsage[weekNumber] = freq.curWeekUsage[weekNumber] || 0;
      freq.curWeekUsage[weekNumber]++;
    }
    if (dishFreqInfo) {
      dishFreqInfo.month++;
      dishFreqInfo.week[weekNumber]++;
      let freq = Frequency.frequencies[dishFreqInfo.idx];
      freq.cur = freq.cur || 0;
      freq.cur++;
      freq.curWeekUsage = freq.curWeekUsage || {};
      freq.curWeekUsage[weekNumber] = freq.curWeekUsage[weekNumber] || 0;
      freq.curWeekUsage[weekNumber]++;
    }
  }

  addFrequencyOnMenu(frequencies, dish, weekNumber) {
    const allFreq = frequencies.filter((f) => {
      return f.COD_PRATO == dish.prato ||
             f.NUM_GRP_PRATO == dish.gpprato ||
            (f.COD_PRODUTO &&
              dish.gpprato == f.NRGRUPOPRATOSERV &&
              dish.prods.find((pr) => pr.COD_PRODUTO == f.COD_PRODUTO));
    });
    if (allFreq) {
      allFreq.forEach((fr) => {
        fr.cur = fr.cur ? fr.cur + 1 : 1;
        fr.curWeekUsage = fr.curWeekUsage || {};
        const weekUsageCount = fr.curWeekUsage[weekNumber] || 0;
        fr.curWeekUsage[weekNumber] = weekUsageCount + 1;
      });
    }
  }

  hasAvailableFrequencyInGroup(groupFrequency, curWeek) {
    const { cur = 0, curWeekUsage = {}, MAX_MES_GRP, MAX_SEM_GRP } = groupFrequency;
    return cur < MAX_MES_GRP && (curWeekUsage[curWeek] || 0) < MAX_SEM_GRP;
  }

  hasAvailableFrequencyInDish(dishFrequency, curWeek) {
    const { cur = 0, curWeekUsage = {}, MAX_MES_PRATO, MAX_SEMANA } = dishFrequency;
    return cur < MAX_MES_PRATO && (curWeekUsage[curWeek] || 0) < MAX_SEMANA;
  }
}

module.exports = new Frequency();
