const Cost = require("./Cost");
const FlexRules = require("./FlexRules");
const Frequency = require("./Frequency");
const MenuAnalytics = require("./MenuAnalytics");
const { getWeek } = require('../Utils/DateUtil');

class MenuReview {
  execute(menu, { restComb, calendar, utilizationGroupWeek }) {
    MenuAnalytics.data = menu.analytics;
    Frequency.load(menu.frequencias);
    menu.comBuraco = 0;
    menu.tabela.log.flexRules = {};
    menu.tabela.pratos.forEach((dishesOnDay, dayNumber) => {
      if(dayNumber === 0) {
        return;
      }
      const curDay = dayNumber.toString().padStart(2, '0');
      const menuStructure = menu.cardapio.datas.find(menuOnDay => menuOnDay.dia && menuOnDay.dia.startsWith(curDay));
      if(!menuStructure) return;
      const { comensais, comenDIV } = menuStructure;
      const consumers = parseInt((comensais || 0), 10);
      let hasEmptyColumn = false;
      let sortDishes = false;
      dishesOnDay.forEach(dish => {
        if(!dish.prato && dish.pratosBKP && dish.pratosBKP.length && !dish.buracoObrigatorio) {
          const { dish: selectedDish, rules } = FlexRules.analyzeMenu(
            dish.data,
            dish.sequencial,
            {
              restComb,
              otherDishes: dish.pratosBKP,
              todayMenu: dishesOnDay,
              calendar,
              utilizationGroupWeek,
            }
          );
          if(selectedDish){
            const dishIdx = dish.pratosBKP.findIndex(curDish => curDish.prato === selectedDish.prato);
            dish.pratosBKP.splice(dishIdx, 1);
            const otherOptions = dish.pratosBKP.length ? [1] : [];
            Object.assign(dish, JSON.parse(JSON.stringify(selectedDish)), { ignoredRules: rules, otherOptions });
            menuStructure.pratos.push(JSON.parse(JSON.stringify(dish)));

            dish.tipoPrato = dish.COD_TIP_PRATO;
            dish.unidadePrato = dish.prato && dish.prato[0] == 'P' ? 'P' : menu.cardapio.unidade;
            dish.codprato = dish.prato;
            dish.prato = dish.unidadePrato == 'P' ? (dish.prato || '').substr(1) : (dish.prato || '').replace(menu.cardapio.unidade, '');
            dish.nrPratoCardapio = dish.sequencial;

            sortDishes = true;
            Frequency.addFrequency(dish, getWeek(dish.data));
            menu.tabela.log.flexRules[dish.data] = menu.tabela.log.flexRules[dish.data] || {};
            menu.tabela.log.flexRules[dish.data][dish.sequencial] = rules;
          } else {
            hasEmptyColumn = true;
          }
        }
        if(dish.prato) {
          Cost.calculate(dish, { comensais, comenDIV });
        }
      });
      if(hasEmptyColumn) {
        menu.comBuraco++;
      }
      if(sortDishes) {
        menuStructure.pratos.sort((dishA, dishB) => Number(dishA.sequencial) < Number(dishB.sequencial) ? -1 : 1);
      }
      Cost.totalizeDay(consumers);
      menuStructure.mediaCusto = Cost.costAvg;
      menu.tabela.pudia_puacu[dayNumber] = [Cost.dayPu, Cost.getTotalPu()];
      Cost.nextDay();
    });
  }
}

module.exports = new MenuReview();
